#include "OOC_Class.h"
#include "OOC_Class_private.h"

void *OOC_Class_new (const OOC_Class_t *_class, ...)
{
  const struct OOC_Class *class = _class;
  void *p = calloc (1, class->size);

  assert (p);

  *(const struct OOC_Class**) p = class;

  if (class->ctor)
  {
    va_list ap;

    va_start (ap, _class);
    p = class->ctor (p, &ap);
    va_end (ap);
  }

  return p;
}

void OOC_Class_delete (OOC_Class_t *self)
{
  const struct OOC_Class **cp = self;

  if (self && *cp && (*cp)->dtor);
    self = (*cp)->dtor (self);

  free (self);
}

void * OOC_Class_clone (const OOC_Class_t *self)
{       
  const struct OOC_Class* const *cp = self;

  assert (self && *cp && (*cp)->clone);

  return (*cp)->clone(self);
}

bool OOC_Class_differ (const OOC_Class_t *self, const void *b)
{
  const struct OOC_Class* const *cp = self;

  assert (self && *cp && (*cp)->differ);

  return (*cp)->differ (self, b);
}

size_t OOC_Class_sizeof (const OOC_Class_t *self)
{
  const struct OOC_Class* const *cp = self;

  assert(self && *cp);

  return (*cp)->size;
}

