#include <string.h>

#include "OOC_String.h"
#include "OOC_String_private.h"

#include "OOC_Class.h"
#include "OOC_Class_private.h"

static OOC_String_t *OOC_String_ctor (OOC_String_t * _self, va_list * app)
{	
  struct OOC_String *self = _self;
	const char *text = va_arg (*app, const char*);

	self->text = malloc (strlen (text) + 1);
	assert (self->text);
	strcpy (self->text, text);

	return self;
}

static OOC_String_t *OOC_String_dtor (OOC_String_t *_self)
{	
  struct OOC_String *self = _self;

	free (self->text);
  self->text = 0;

	return self;
}

static OOC_String_t *OOC_String_clone (const OOC_String_t *_self)
{	
  const struct OOC_String *self = _self;

	return OOC_Class_new (OOC_String, self->text);
}

static bool OOC_String_differ (const OOC_String_t *_self, const void *_b)
{	
  const struct OOC_String *self = _self;
	const struct OOC_String *b = _b;

	if (self == b)
		return 0;

	if (! b || b->class != OOC_String)
		return 1;

	return strcmp (self->text, b->text);
}

// define inherit functions from OOC_Class
static const struct OOC_Class _OOC_String = {
	sizeof(struct OOC_String),
	OOC_String_ctor, OOC_String_dtor,
	OOC_String_clone, OOC_String_differ
};

const OOC_String_t *OOC_String = &_OOC_String;

/*** new functions ***/

const char *OOC_String_c_str (const OOC_String_t *_self)
{
  const struct OOC_String *self = _self;

  return self->text;
}

void OOC_String_append_ooc (OOC_String_t *_self, const void *_b)
{	
  struct OOC_String *self = _self;
	const struct OOC_String *b = _b;

	if (self == b)
		return;

	if (!b || b->class != OOC_String)
		return;

	void *mem = realloc ((void*)self->text, strlen (self->text) + strlen (b->text) + 1);
  assert (mem);
  self->text = mem;
  
  strcat (self->text, b->text);
}

void OOC_String_append (OOC_String_t *_self, const char *str)
{
  struct OOC_String *self = _self;

  void *mem = realloc ((void*)self->text, strlen (self->text) + strlen (str) + 1);
  assert (mem);
  self->text = mem;
  
  strcat (self->text, str);
}

void OOC_String_assign (OOC_String_t *_self, const char *str)
{
  struct OOC_String *self = _self;

  void *mem = realloc ((void*)self->text, strlen (str) + 1);
  assert (mem);
  self->text = mem;
  
  strcpy (self->text, str);
}

void OOC_String_assign_to (OOC_String_t *_self, const char *str, const size_t to)
{
  struct OOC_String *self = _self;

  void *mem = realloc ((void*)self->text, to);
  assert (mem);
  self->text = mem;
  
  strncpy (self->text, str, to);
}

void OOC_String_erase (OOC_String_t *_self)
{
  struct OOC_String *self = _self;

  void *mem = realloc ((OOC_String_t*) self->text, 1);
  assert (mem);
  self->text = mem;
  
  self->text[0] = '\0';
}

bool OOC_String_empty (const OOC_String_t *_self)
{
  const struct OOC_String *self = _self;

  return !strlen (self->text);
}

int OOC_String_compare (const OOC_String_t *_self, const char* str)
{
  const struct OOC_String *self = _self;

  return strcmp (self->text, str);
}

size_t OOC_String_length (const OOC_String_t *_self)
{
  const struct OOC_String *self = _self;

  return strlen (self->text);
}

char *OOC_String_at (OOC_String_t *_self, size_t loc)
{
  struct OOC_String *self = _self;
  
  if (OOC_String_length (self) < loc)
    return 0; // should be exception/errno because loc is to big!

  return &self->text[loc];
}

const char *OOC_String_at_const (const OOC_String_t *_self, size_t loc)
{
  const struct OOC_String *self = _self;

  return &self->text[loc];
}

